@@ -29,15 +29,15 @@
 
 #include "base/file_util.h"
 
+#include <ios>
 #include <string>
 
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
 #include "base/file/temp_dir.h"
-#include "base/logging.h"
 #include "testing/gmock.h"
 #include "testing/gunit.h"
 #include "testing/mozctest.h"
-#include "absl/status/status.h"
-#include "absl/strings/str_format.h"
 
 #ifdef _WIN32
 #include <windows.h>
@@ -45,7 +45,7 @@
 #include "base/win32/wide_char.h"
 #endif  // _WIN32
 
-// Ad-hoc workadound against macro problem on Windows.
+// Ad-hoc workaround against macro problem on Windows.
 // On Windows, following macros, defined when you include <windows.h>,
 // should be removed here because they affects the method name definition of
 // Util class.
@@ -74,6 +74,15 @@ TEST(FileUtilTest, CreateDirectory) {
   EXPECT_OK(FileUtil::CreateDirectory(dirpath));
   EXPECT_OK(FileUtil::DirectoryExists(dirpath));
 
+#if !defined(_WIN32)
+  // On Windows, CreateDirectory does not return OK if the directory already
+  // exists. See the implementation of CreateDirectory in file_util.cc.
+  // https://github.com/google/mozc/issues/1076
+  //
+  // Create the same directory again.
+  EXPECT_OK(FileUtil::CreateDirectory(dirpath));
+#endif  // !_WIN32
+
   // Delete the directory.
   ASSERT_OK(FileUtil::RemoveDirectory(dirpath));
   ASSERT_FALSE(FileUtil::FileExists(dirpath).ok());
@@ -491,36 +500,39 @@ TEST(FileUtilTest, GetAndSetContents) {
   const std::string filename = FileUtil::JoinPath(temp_dir.path(), "test.txt");
 
   // File doesn't exist yet.
-  std::string content;
-  EXPECT_TRUE(absl::IsNotFound(FileUtil::GetContents(filename, &content)));
+  EXPECT_TRUE(absl::IsNotFound(FileUtil::GetContents(filename).status()));
 
   // Basic write and read test.
   ASSERT_OK(FileUtil::SetContents(filename, "test"));
   FileUnlinker unlinker(filename);
-  EXPECT_OK(FileUtil::GetContents(filename, &content));
-  EXPECT_EQ(content, "test");
+  absl::StatusOr<std::string> content = FileUtil::GetContents(filename);
+  EXPECT_OK(content);
+  EXPECT_EQ(*content, "test");
 
   // Overwrite test.
   ASSERT_OK(FileUtil::SetContents(filename, "more tests!"));
-  EXPECT_OK(FileUtil::GetContents(filename, &content));
-  EXPECT_EQ(content, "more tests!");
+  content = FileUtil::GetContents(filename);
+  EXPECT_OK(content);
+  EXPECT_EQ(*content, "more tests!");
 
   // Text mode write.
   ASSERT_OK(FileUtil::SetContents(filename, "test\ntest\n", std::ios::out));
-  EXPECT_OK(FileUtil::GetContents(filename, &content));
+  content = FileUtil::GetContents(filename);
+  EXPECT_OK(content);
 #ifdef _WIN32
-  EXPECT_EQ(content, "test\r\ntest\r\n");
+  EXPECT_EQ(*content, "test\r\ntest\r\n");
 #else   // _WIN32
-  EXPECT_EQ(content, "test\ntest\n");
+  EXPECT_EQ(*content, "test\ntest\n");
 #endif  // _WIN32
 
   // Text mode read.
   ASSERT_OK(FileUtil::SetContents(filename, "test\r\ntest\r\n"));
-  EXPECT_OK(FileUtil::GetContents(filename, &content, std::ios::in));
+  content = FileUtil::GetContents(filename, std::ios::in);
+  EXPECT_OK(content);
 #ifdef _WIN32
-  EXPECT_EQ(content, "test\ntest\n");
+  EXPECT_EQ(*content, "test\ntest\n");
 #else   // _WIN32
-  EXPECT_EQ(content, "test\r\ntest\r\n");
+  EXPECT_EQ(*content, "test\r\ntest\r\n");
 #endif  // _WIN32
 }
 
@@ -536,19 +548,5 @@ TEST(FileUtilTest, FileUnlinker) {
   EXPECT_FALSE(FileUtil::FileExists(temp_file->path()).ok());
 }
 
-TEST(FileUtilTest, LinkOrCopyFile) {
-  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
-  const std::string from =
-      FileUtil::JoinPath(temp_dir.path(), "link_or_copy_test_from.txt");
-  const std::string to =
-      FileUtil::JoinPath(temp_dir.path(), "link_or_copy_test_to.txt");
-  EXPECT_TRUE(!FileUtil::LinkOrCopyFile(from, to).ok());
-  ASSERT_OK(FileUtil::SetContents(from, "test"));
-  EXPECT_OK(FileUtil::LinkOrCopyFile(from, to));
-  auto s = FileUtil::IsEqualFile(from, to);
-  EXPECT_OK(s);
-  EXPECT_TRUE(*s);
-}
-
 }  // namespace
 }  // namespace mozc
