@@ -39,6 +39,7 @@ Typical usage:
 
 import glob
 import logging
+import multiprocessing
 import optparse
 import os
 import pathlib
@@ -61,6 +62,7 @@ from build_tools.util import RemoveDirectoryRecursively
 from build_tools.util import RemoveFile
 from build_tools.util import RunOrDie
 from build_tools.util import RunOrDieError
+from build_tools.vs_util import get_vs_env_vars
 
 SRC_DIR = '.'
 OSS_SRC_DIR = '.'
@@ -152,11 +154,11 @@ def GetGypFileNames(options):
       glob.glob('%s/data/test/session/scenario/*/*.gyp' % OSS_SRC_DIR)
   )
   # Include subdirectories of data_manager
-  gyp_file_names.extend(glob.glob('%s/data_manager/*/*.gyp' % SRC_DIR))
+  gyp_file_names.extend(glob.glob('%s/data_manager/*/*.gyp' % OSS_SRC_DIR))
   # Include subdirectory of dictionary
-  gyp_file_names.extend(glob.glob('%s/dictionary/*/*.gyp' % SRC_DIR))
+  gyp_file_names.extend(glob.glob('%s/dictionary/*/*.gyp' % OSS_SRC_DIR))
   # Include subdirectory of rewriter
-  gyp_file_names.extend(glob.glob('%s/rewriter/*/*.gyp' % SRC_DIR))
+  gyp_file_names.extend(glob.glob('%s/rewriter/*/*.gyp' % OSS_SRC_DIR))
   # Include subdirectory of win32 and breakpad for Windows
   if options.target_platform == 'Windows':
     gyp_file_names.extend(glob.glob('%s/win32/*/*.gyp' % OSS_SRC_DIR))
@@ -193,15 +195,6 @@ def AddTargetPlatformOption(parser):
                           'should be done.'))
 
 
-def GetDefaultWixPath():
-  """Returns the default Wix directory.."""
-  possible_wix_path = pathlib.Path(ABS_SCRIPT_DIR).joinpath(
-      'third_party', 'wix')
-  if possible_wix_path.exists():
-    return possible_wix_path
-  return ''
-
-
 def GetDefaultQtPath():
   """Returns the default Qt directory.."""
   qtdir_env = os.getenv('QTDIR', None)
@@ -260,9 +253,8 @@ def ParseGypOptions(args):
     parser.add_option('--msvs_version', dest='msvs_version',
                       default='2022',
                       help='Version of the Visual Studio.')
-    parser.add_option('--wix_dir', dest='wix_dir',
-                      default=GetDefaultWixPath(),
-                      help='A path to the binary directory of wix.')
+    parser.add_option('--vcvarsall_path', help='Path of vcvarsall.bat',
+                      default=None)
 
   if IsWindows() or IsMac():
     parser.add_option('--qtdir', dest='qtdir',
@@ -297,7 +289,7 @@ def ExpandMetaTarget(options, meta_target_name):
 
   if target_platform == 'Linux':
     targets = [
-        SRC_DIR + '/server/server.gyp:mozc_server',
+        OSS_SRC_DIR + '/server/server.gyp:mozc_server',
         OSS_SRC_DIR + '/gui/gui.gyp:mozc_tool',
     ]
   elif target_platform == 'Mac':
@@ -309,6 +301,8 @@ def ExpandMetaTarget(options, meta_target_name):
         'out_win/%s_x64:mozc_win32_build64' % config,
         'out_win/%s:mozc_installers_win' % config,
     ]
+  else:
+    raise ValueError('unrecognized platform: %s' % target_platform)
 
   return dependencies + targets
 
@@ -355,6 +349,27 @@ def ParseCleanOptions(args):
   return parser.parse_args(args)
 
 
+def ReadEnvironmentFile(path):
+  nul = chr(0)
+  with open(path, 'rb') as f:
+    content = f.read()
+  entries = content.decode('utf-8').split(nul)
+  env = dict()
+  for e in entries:
+    if '=' in e:
+      key, value = e.split('=', 1)
+      env[key] = value
+  return env
+
+
+def WriteEnvironmentFile(path, env):
+  nul = chr(0)
+  entries = [f'{key}={value}' for (key, value) in env.items()]
+  entries.extend(['', ''])
+  with open(path, 'wb') as f:
+    f.write(nul.join(entries).encode('utf-8'))
+
+
 def UpdateEnvironmentFilesForWindows(out_dir):
   """Add required environment variables for Ninja build."""
   python_path_root = MOZC_ROOT
@@ -362,26 +377,27 @@ def UpdateEnvironmentFilesForWindows(out_dir):
   original_python_paths = os.environ.get('PYTHONPATH', '')
   if original_python_paths:
     python_path = os.pathsep.join([original_python_paths, python_path])
-  nul = chr(0)
-  additional_content = nul.join([
-      'PYTHONPATH=' + python_path,
-      'VSLANG=1033',  # 1033 == MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US)
-      nul]).encode('utf-8')
   for d in os.listdir(out_dir):
     abs_dir = os.path.abspath(os.path.join(out_dir, d))
-    with open(os.path.join(abs_dir, 'environment.x86'), 'rb') as x86_file:
-      x86_content = x86_file.read()[:-1] + additional_content
-    with open(os.path.join(abs_dir, 'environment.x86'), 'wb') as x86_file:
-      x86_file.write(x86_content)
-    with open(os.path.join(abs_dir, 'environment.x64'), 'rb') as x64_file:
-      x64_content = x64_file.read()[:-1] + additional_content
-    with open(os.path.join(abs_dir, 'environment.x64'), 'wb') as x64_file:
-      x64_file.write(x64_content)
-
+    for arch in ['x86', 'x64']:
+      env_file = os.path.join(abs_dir, f'environment.{arch}')
+      env = ReadEnvironmentFile(env_file)
+      env['PYTHONPATH'] = python_path
+      env['VSLANG'] = '1033' # == MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US)
+      WriteEnvironmentFile(env_file, env)
 
 
 def GypMain(options, unused_args):
   """The main function for the 'gyp' command."""
+  if IsWindows():
+    # GYP captures environment variables while running so setting them up as if
+    # the developer manually executed 'vcvarsall.bat' command. Otherwise we end
+    # up having to explain how to do that for both cmd.exe and PowerShell.
+    # See https://github.com/google/mozc/pull/923
+    env_vars = get_vs_env_vars('x64', options.vcvarsall_path)
+    for (key, value) in env_vars.items():
+      os.environ[key] = value
+
   # Generate a version definition file.
   logging.info('Generating version definition file...')
   template_path = '%s/%s' % (OSS_SRC_DIR, options.version_file)
@@ -452,9 +468,9 @@ def GypMain(options, unused_args):
   logging.info('Building GYP command line...')
   gyp_options = ['--depth=.']
   if target_platform == 'Windows':
-    gyp_options.extend(['--include=%s/gyp/common_win.gypi' % SRC_DIR])
+    gyp_options.extend(['--include=%s/gyp/common_win.gypi' % OSS_SRC_DIR])
   else:
-    gyp_options.extend(['--include=%s/gyp/common.gypi' % SRC_DIR])
+    gyp_options.extend(['--include=%s/gyp/common.gypi' % OSS_SRC_DIR])
 
   gyp_options.extend(['-D', 'abs_depth=%s' % MOZC_ROOT])
   gyp_options.extend(['-D', 'ext_third_party_dir=%s' % EXT_THIRD_PARTY_DIR])
@@ -494,9 +510,8 @@ def GypMain(options, unused_args):
   gyp_options.extend(['-D', 'qt_dir=' + (qt_dir or '')])
   gyp_options.extend(['-D', 'qt_ver=' + str(qt_ver or '')])
 
-  if target_platform == 'Windows' and options.wix_dir:
+  if target_platform == 'Windows':
     gyp_options.extend(['-D', 'use_wix=YES'])
-    gyp_options.extend(['-D', 'wix_dir="%s"' % options.wix_dir])
   else:
     gyp_options.extend(['-D', 'use_wix=NO'])
 
@@ -526,10 +541,11 @@ def GypMain(options, unused_args):
   short_basename = GetBuildShortBaseName(target_platform)
   gyp_options.extend(['-G', 'output_dir=%s' % short_basename])
 
-  # Enable cross-compile
-  # TODO(yukawa): Enable GYP_CROSSCOMPILE for Windows.
-  if not IsWindows():
-    os.environ['GYP_CROSSCOMPILE'] = '1'
+  if IsWindows() and multiprocessing.cpu_count() > 63:
+    # GYP fails on Windows with more than 63 cores:
+    #   "ValueError: need at most 63 handles"
+    # GYP doesn't have an option to set max threads, just disable parallel.
+    gyp_options.append('--no-parallel')
 
   # Run GYP.
   logging.info('Running GYP...')
@@ -796,7 +812,7 @@ def RunTestsMain(options, args):
     if target_platform == 'Windows':
       targets.append('out_win/%sDynamic_x64:unittests' % options.configuration)
     else:
-      targets.append('%s/gyp/tests.gyp:unittests' % SRC_DIR)
+      targets.append('%s/gyp/tests.gyp:unittests' % OSS_SRC_DIR)
 
   # Build the test targets
   (build_opts, build_args) = ParseBuildOptions(build_options + targets)
@@ -872,11 +888,6 @@ def main():
   command = sys.argv[1]
   args = sys.argv[2:]
 
-  if IsWindows() and (not os.environ.get('VCToolsRedistDir', '')):
-    print('VCToolsRedistDir is not defined.')
-    print('Please use Developer Command Prompt or run vcvarsamd64_x86.bat')
-    return 1
-
   if command == 'gyp':
     (cmd_opts, cmd_args) = ParseGypOptions(args)
     GypMain(cmd_opts, cmd_args)
