@@ -29,13 +29,7 @@
 
 #include "config/stats_config_util.h"
 
-#include <string>
-
-#include "base/singleton.h"
 #include "testing/gunit.h"
-#include "absl/base/casts.h"
-#include "absl/container/flat_hash_map.h"
-#include "absl/flags/flag.h"
 
 #ifdef __ANDROID__
 #include "base/file/temp_dir.h"
@@ -45,6 +39,10 @@
 #endif  // __ANDROID__
 
 #ifdef _WIN32
+#include <bit>
+
+#include "absl/container/flat_hash_map.h"
+#include "base/singleton.h"
 #include "base/win32/win_api_test_helper.h"
 #endif  // _WIN32
 
@@ -64,8 +62,7 @@ constexpr wchar_t kOmahaUsageKeyForEveryone[] =
     L"{DDCCD2A9-025E-4142-BCEB-F467B88CF830}";
 constexpr wchar_t kSendStatsName[] = L"usagestats";
 
-// TODO(yuryu): absl::bit_cast is not constexpr with VC++2017.
-HKEY DefineHKey(const uintptr_t value) { return absl::bit_cast<HKEY>(value); }
+HKEY DefineHKey(const uintptr_t value) { return std::bit_cast<HKEY>(value); }
 
 const HKEY kHKCU_ClientState = DefineHKey(1);
 const HKEY kHKLM_ClientState = DefineHKey(2);
@@ -124,7 +121,7 @@ class RegistryEmulator {
       }
       return i->second;
     }
-    absl::flat_hash_map<HKEY, DWORD> &usagestats_map() const {
+    const absl::flat_hash_map<HKEY, DWORD> &usagestats_map() const {
       return usagestats_map_;
     }
     int run_level() const { return run_level_; }
@@ -679,8 +676,8 @@ TEST(StatsConfigUtilTestWin, IsEnabled) {
 #ifdef __ANDROID__
 TEST(StatsConfigUtilTestAndroid, DefaultValueTest) {
   const TempFile config_file(testing::MakeTempFileOrDie());
-  ConfigHandler::SetConfigFileName(config_file.path());
-  EXPECT_EQ(ConfigHandler::GetConfigFileName(), config_file.path());
+  ConfigHandler::SetConfigFileNameForTesting(config_file.path());
+  EXPECT_EQ(ConfigHandler::GetConfigFileNameForTesting(), config_file.path());
   ConfigHandler::Reload();
 #ifdef CHANNEL_DEV
   EXPECT_TRUE(StatsConfigUtil::IsEnabled());
