@@ -39,18 +39,20 @@
 #include <string>
 #include <utility>
 
-#include "base/logging.h"
+#include "absl/log/check.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "base/number_util.h"
 #include "base/util.h"
+#include "base/vlog.h"
 #include "converter/node.h"
 #include "converter/segments.h"
+#include "dictionary/dictionary_interface.h"
 #include "dictionary/pos_matcher.h"
-#include "dictionary/suppression_dictionary.h"
 #include "prediction/suggestion_filter.h"
 #include "protocol/commands.pb.h"
 #include "request/conversion_request.h"
-#include "absl/strings/string_view.h"
-#include "absl/types/span.h"
+#include "request/request_util.h"
 
 namespace mozc {
 namespace converter {
@@ -58,7 +60,7 @@ namespace {
 
 using ::mozc::converter::candidate_filter_internal::CandidateId;
 using ::mozc::dictionary::PosMatcher;
-using ::mozc::dictionary::SuppressionDictionary;
+using ::mozc::dictionary::UserDictionaryInterface;
 
 constexpr size_t kSizeThresholdForWeakCompound = 10;
 
@@ -92,7 +94,7 @@ constexpr int32_t kStopEnmerationCacheSize = 30;
 // Returns true if the given node sequence is noisy weak compound.
 // Please refer to the comment in FilterCandidateInternal for the idea.
 inline bool IsNoisyWeakCompound(const absl::Span<const Node *const> nodes,
-                                const dictionary::PosMatcher *pos_matcher) {
+                                const dictionary::PosMatcher &pos_matcher) {
   if (nodes.size() <= 1) {
     return false;
   }
@@ -100,7 +102,7 @@ inline bool IsNoisyWeakCompound(const absl::Span<const Node *const> nodes,
     // nodes[0] is COMPOUND entry in dictionary.
     return false;
   }
-  if (pos_matcher->IsWeakCompoundFillerPrefix(nodes[0]->lid)) {
+  if (pos_matcher.IsWeakCompoundFillerPrefix(nodes[0]->lid)) {
     // Word that starts with 'filler' is always noisy.
     return true;
   }
@@ -108,13 +110,13 @@ inline bool IsNoisyWeakCompound(const absl::Span<const Node *const> nodes,
     // Some node +  COMPOUND node may be noisy.
     return true;
   }
-  if (pos_matcher->IsWeakCompoundNounPrefix(nodes[0]->lid) &&
-      !pos_matcher->IsWeakCompoundNounSuffix(nodes[1]->lid)) {
+  if (pos_matcher.IsWeakCompoundNounPrefix(nodes[0]->lid) &&
+      !pos_matcher.IsWeakCompoundNounSuffix(nodes[1]->lid)) {
     // Noun prefix + not noun
     return true;
   }
-  if (pos_matcher->IsWeakCompoundVerbPrefix(nodes[0]->lid) &&
-      !pos_matcher->IsWeakCompoundVerbSuffix(nodes[1]->lid)) {
+  if (pos_matcher.IsWeakCompoundVerbPrefix(nodes[0]->lid) &&
+      !pos_matcher.IsWeakCompoundVerbSuffix(nodes[1]->lid)) {
     // Verb prefix + not verb
     return true;
   }
@@ -124,7 +126,7 @@ inline bool IsNoisyWeakCompound(const absl::Span<const Node *const> nodes,
 // Returns true if the given node sequence is connected weak compound.
 // Please refer to the comment in FilterCandidateInternal for the idea.
 inline bool IsConnectedWeakCompound(const absl::Span<const Node *const> nodes,
-                                    const dictionary::PosMatcher *pos_matcher) {
+                                    const dictionary::PosMatcher &pos_matcher) {
   if (nodes.size() <= 1) {
     return false;
   }
@@ -132,13 +134,13 @@ inline bool IsConnectedWeakCompound(const absl::Span<const Node *const> nodes,
     // nodes[0/1] is COMPOUND entry in dictionary.
     return false;
   }
-  if (pos_matcher->IsWeakCompoundNounPrefix(nodes[0]->lid) &&
-      pos_matcher->IsWeakCompoundNounSuffix(nodes[1]->lid)) {
+  if (pos_matcher.IsWeakCompoundNounPrefix(nodes[0]->lid) &&
+      pos_matcher.IsWeakCompoundNounSuffix(nodes[1]->lid)) {
     // Noun prefix + noun
     return true;
   }
-  if (pos_matcher->IsWeakCompoundVerbPrefix(nodes[0]->lid) &&
-      pos_matcher->IsWeakCompoundVerbSuffix(nodes[1]->lid)) {
+  if (pos_matcher.IsWeakCompoundVerbPrefix(nodes[0]->lid) &&
+      pos_matcher.IsWeakCompoundVerbSuffix(nodes[1]->lid)) {
     // Verb prefix + verb
     return true;
   }
@@ -214,25 +216,45 @@ bool IsSameNodeStructure(const absl::Span<const Node *const> lnodes,
   return true;
 }
 
-bool IsStrictModeEnabled(const ConversionRequest &request) {
-  return request.request().mixed_conversion() &&
-         !request.request()
-              .decoder_experiment_params()
-              .enable_realtime_conversion_v2();
+// Returns true if there is a number node that does not follow the
+bool IsNoisyNumberCandidate(const dictionary::PosMatcher &pos_matcher,
+                            const absl::Span<const Node *const> nodes) {
+  auto is_converted_number = [&](const Node &node) {
+    if (node.lid != node.rid) {
+      return false;
+    }
+    if (!Util::IsScriptType(node.key, Util::HIRAGANA)) {
+      return false;
+    }
+    return pos_matcher.IsNumber(node.lid) ||
+           pos_matcher.IsKanjiNumber(node.rid);
+  };
+  for (int i = 0; i < nodes.size(); ++i) {
+    if (!is_converted_number(*nodes[i])) {
+      continue;
+    }
+    if (i + 1 < nodes.size() && !is_converted_number(*nodes[i + 1]) &&
+        !pos_matcher.IsCounterSuffixWord(nodes[i + 1]->lid)) {
+      // "にいく": "2行く"
+      return true;
+    }
+    if (i - 1 >= 0 && pos_matcher.IsUniqueNoun(nodes[i - 1]->rid)) {
+      // "しんじゅくに": "新宿2"
+      return true;
+    }
+  }
+  return false;
 }
 
 }  // namespace
 
-CandidateFilter::CandidateFilter(
-    const SuppressionDictionary *suppression_dictionary,
-    const PosMatcher *pos_matcher, const SuggestionFilter &suggestion_filter)
-    : suppression_dictionary_(suppression_dictionary),
+CandidateFilter::CandidateFilter(const UserDictionaryInterface &user_dictionary,
+                                 const PosMatcher &pos_matcher,
+                                 const SuggestionFilter &suggestion_filter)
+    : user_dictionary_(user_dictionary),
       pos_matcher_(pos_matcher),
       suggestion_filter_(suggestion_filter),
-      top_candidate_(nullptr) {
-  CHECK(suppression_dictionary_);
-  CHECK(pos_matcher_);
-}
+      top_candidate_(nullptr) {}
 
 void CandidateFilter::Reset() {
   seen_.clear();
@@ -310,8 +332,6 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
     return result;
   }
 
-  const bool is_strict_mode = IsStrictModeEnabled(request);
-
   // In general, the cost of constrained node tends to be overestimated.
   // If the top candidate has constrained node, we skip the main body
   // of CandidateFilter, meaning that the node is not treated as the top
@@ -320,6 +340,12 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
     return CandidateFilter::GOOD_CANDIDATE;
   }
 
+  if (request_util::ShouldFilterNoisyNumberCandidate(request)) {
+    if (IsNoisyNumberCandidate(pos_matcher_, nodes)) {
+      return CandidateFilter::BAD_CANDIDATE;
+    }
+  }
+
   const size_t candidate_size = seen_.size();
   if (top_candidate_ == nullptr || candidate_size == 0) {
     top_candidate_ = candidate;
@@ -330,13 +356,13 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   // "短縮よみ" or "記号,一般" must have only 1 node.  Note that "顔文字" POS
   // from user dictionary is converted to "記号,一般" in Mozc engine.
   if (nodes.size() > 1 &&
-      ContainsIsolatedWordOrGeneralSymbol(*pos_matcher_, nodes)) {
+      ContainsIsolatedWordOrGeneralSymbol(pos_matcher_, nodes)) {
     MOZC_CANDIDATE_LOG(candidate, "ContainsIsolatedWordOrGeneralSymbol");
     return CandidateFilter::BAD_CANDIDATE;
   }
   // This case tests the case where the isolated word or general symbol is in
   // content word.
-  if (IsIsolatedWordOrGeneralSymbol(*pos_matcher_, nodes[0]->lid) &&
+  if (IsIsolatedWordOrGeneralSymbol(pos_matcher_, nodes[0]->lid) &&
       (IsNormalOrConstrainedNode(nodes[0]->prev) ||
        IsNormalOrConstrainedNode(nodes[0]->next))) {
     MOZC_CANDIDATE_LOG(candidate, "IsIsolatedWordOrGeneralSymbol");
@@ -344,12 +370,11 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   }
 
   // Remove "抑制単語" just in case.
-  if (suppression_dictionary_->SuppressEntry(candidate->key,
-                                             candidate->value) ||
+  if (user_dictionary_.IsSuppressedEntry(candidate->key, candidate->value) ||
       (candidate->key != candidate->content_key &&
        candidate->value != candidate->content_value &&
-       suppression_dictionary_->SuppressEntry(candidate->content_key,
-                                              candidate->content_value))) {
+       user_dictionary_.IsSuppressedEntry(candidate->content_key,
+                                          candidate->content_value))) {
     MOZC_CANDIDATE_LOG(candidate, "SuppressEntry");
     return CandidateFilter::BAD_CANDIDATE;
   }
@@ -383,15 +408,15 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   if (Util::GetScriptType(nodes[0]->value) != Util::HIRAGANA) {
     if (nodes.size() >= 2) {
       // For node sequence
-      if (pos_matcher_->IsKagyoTaConnectionVerb(nodes[0]->rid) &&
-          pos_matcher_->IsVerbSuffix(nodes[1]->lid) &&
-          !pos_matcher_->IsTeSuffix(nodes[1]->lid)) {
+      if (pos_matcher_.IsKagyoTaConnectionVerb(nodes[0]->rid) &&
+          pos_matcher_.IsVerbSuffix(nodes[1]->lid) &&
+          !pos_matcher_.IsTeSuffix(nodes[1]->lid)) {
         // "書い" | "ます", "過ぎ", etc
         MOZC_CANDIDATE_LOG(candidate, "IsKagyoTaConnectionVerb");
         return CandidateFilter::BAD_CANDIDATE;
       }
-      if (pos_matcher_->IsWagyoRenyoConnectionVerb(nodes[0]->rid) &&
-          pos_matcher_->IsTeSuffix(nodes[1]->lid)) {
+      if (pos_matcher_.IsWagyoRenyoConnectionVerb(nodes[0]->rid) &&
+          pos_matcher_.IsTeSuffix(nodes[1]->lid)) {
         // "買い" | "て"
         MOZC_CANDIDATE_LOG(candidate, "IsWagyoRenyoConnectionVerb");
         return CandidateFilter::BAD_CANDIDATE;
@@ -399,15 +424,15 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
     }
     if (nodes[0]->lid != nodes[0]->rid) {
       // For compound
-      if (pos_matcher_->IsKagyoTaConnectionVerb(nodes[0]->lid) &&
-          pos_matcher_->IsVerbSuffix(nodes[0]->rid) &&
-          !pos_matcher_->IsTeSuffix(nodes[0]->rid)) {
+      if (pos_matcher_.IsKagyoTaConnectionVerb(nodes[0]->lid) &&
+          pos_matcher_.IsVerbSuffix(nodes[0]->rid) &&
+          !pos_matcher_.IsTeSuffix(nodes[0]->rid)) {
         // "書い" | "ます", "過ぎ", etc
         MOZC_CANDIDATE_LOG(candidate, "IsKagyoTaConnectionVerb");
         return CandidateFilter::BAD_CANDIDATE;
       }
-      if (pos_matcher_->IsWagyoRenyoConnectionVerb(nodes[0]->lid) &&
-          pos_matcher_->IsTeSuffix(nodes[0]->rid)) {
+      if (pos_matcher_.IsWagyoRenyoConnectionVerb(nodes[0]->lid) &&
+          pos_matcher_.IsTeSuffix(nodes[0]->rid)) {
         // "買い" | "て"
         MOZC_CANDIDATE_LOG(candidate, "IsWagyoRenyoConnectionVerb");
         return CandidateFilter::BAD_CANDIDATE;
@@ -417,13 +442,13 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
 
   // The candidate consists of only one token
   if (nodes.size() == 1) {
-    VLOG(1) << "don't filter single segment: " << candidate->value;
+    MOZC_VLOG(1) << "don't filter single segment: " << candidate->value;
     return CandidateFilter::GOOD_CANDIDATE;
   }
 
   // don't drop single character
   if (Util::CharsLen(candidate->value) == 1) {
-    VLOG(1) << "don't filter single character: " << candidate->value;
+    MOZC_VLOG(1) << "don't filter single character: " << candidate->value;
     return CandidateFilter::GOOD_CANDIDATE;
   }
 
@@ -459,7 +484,7 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   if (!is_noisy_weak_compound && top_candidate_->structure_cost == 0 &&
       candidate->lid == top_candidate_->lid &&
       candidate->rid == top_candidate_->rid) {
-    VLOG(1) << "don't filter lid/rid are the same:" << candidate->value;
+    MOZC_VLOG(1) << "don't filter lid/rid are the same:" << candidate->value;
     return CandidateFilter::GOOD_CANDIDATE;
   }
 
@@ -476,14 +501,9 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
       top_candidate_->content_value != top_candidate_->value &&
       Util::GetScriptType(top_non_content_value) == Util::HIRAGANA &&
       top_non_content_value == non_content_value) {
-    VLOG(1) << "don't filter if non-content value are the same: "
-            << candidate->value;
-    if (!is_strict_mode || top_nodes.size() == nodes.size()) {
-      // In strict mode, also checks the nodes size.
-      // i.e. do not allow the candidate, "め+移転+も" for the top candidate,
-      // "名店も"
-      return CandidateFilter::GOOD_CANDIDATE;
-    }
+    MOZC_VLOG(1) << "don't filter if non-content value are the same: "
+                 << candidate->value;
+    return CandidateFilter::GOOD_CANDIDATE;
   }
 
   // Check Katakana transliterations
@@ -504,7 +524,7 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
       // nodes[1..] are non-functional candidates.
       // In other words, the node just after KatakanaT13n candidate should
       // be a functional word.
-      if (is_top_english_t13n && !pos_matcher_->IsFunctional(nodes[i]->lid)) {
+      if (is_top_english_t13n && !pos_matcher_.IsFunctional(nodes[i]->lid)) {
         MOZC_CANDIDATE_LOG(candidate, "!IsFunctional");
         return CandidateFilter::BAD_CANDIDATE;
       }
@@ -530,8 +550,8 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
   // We basically ignores the cost threadshould. Filter candidate
   // only by structure cost.
   int cost_offset = kCostOffset;
-  if (candidate->lid == pos_matcher_->GetLastNameId() ||
-      candidate->lid == pos_matcher_->GetFirstNameId()) {
+  if (candidate->lid == pos_matcher_.GetLastNameId() ||
+      candidate->lid == pos_matcher_.GetFirstNameId()) {
     cost_offset = INT_MAX - top_cost;
   }
 
@@ -541,12 +561,12 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
           candidate->structure_cost) {
     // Stops candidates enumeration when we see sufficiently high cost
     // candidate.
-    VLOG(2) << "cost is invalid: "
-            << "top_cost=" << top_cost << " cost_offset=" << cost_offset
-            << " value=" << candidate->value << " cost=" << candidate->cost
-            << " top_structure_cost=" << top_structure_cost
-            << " structure_cost=" << candidate->structure_cost
-            << " lid=" << candidate->lid << " rid=" << candidate->rid;
+    MOZC_VLOG(2) << "cost is invalid: " << "top_cost=" << top_cost
+                 << " cost_offset=" << cost_offset
+                 << " value=" << candidate->value << " cost=" << candidate->cost
+                 << " top_structure_cost=" << top_structure_cost
+                 << " structure_cost=" << candidate->structure_cost
+                 << " lid=" << candidate->lid << " rid=" << candidate->rid;
     if (candidate_size < kStopEnmerationCacheSize) {
       // Even when the current candidate is classified as bad candidate,
       // we don't return STOP_ENUMERATION here.
@@ -572,9 +592,9 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
     // web dictionary entries.
     // For avoiding over filtering, we use kMinStructureCostOffset if
     // |top_structure_cost| is small.
-    VLOG(2) << "structure cost is invalid:  " << candidate->value << " "
-            << candidate->content_value << " " << candidate->structure_cost
-            << " " << candidate->cost;
+    MOZC_VLOG(2) << "structure cost is invalid:  " << candidate->value << " "
+                 << candidate->content_value << " " << candidate->structure_cost
+                 << " " << candidate->cost;
     MOZC_CANDIDATE_LOG(candidate, "structure cost is invalid");
     return CandidateFilter::BAD_CANDIDATE;
   }
@@ -589,19 +609,19 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
     int number_nodes = 0;
     uint16_t prev_lid = 0;
     for (const auto &node : nodes) {
-      absl::string_view value = node->value;
-      size_t mblen = 0;
       if (Util::IsScriptType(node->key, Util::NUMBER)) {
         continue;
       }
-      const Util::ScriptType first_value_script_type = Util::GetScriptType(
-          node->value.data(), node->value.data() + node->value.size(), &mblen);
+      const absl::string_view value = node->value;
+      size_t mblen = 0;
+      const Util::ScriptType first_value_script_type =
+          Util::GetFirstScriptType(value, &mblen);
       if (first_value_script_type == Util::NUMBER && prev_lid != node->lid) {
         ++number_nodes;
       } else if (first_value_script_type == Util::KANJI) {
-        const auto first_kanji = value.substr(0, mblen);
-        std::string converted;
-        NumberUtil::KanjiNumberToArabicNumber(first_kanji, &converted);
+        const absl::string_view first_kanji = value.substr(0, mblen);
+        const std::string converted =
+            NumberUtil::KanjiNumberToArabicNumber(first_kanji);
         if (first_kanji != converted && prev_lid != node->lid) {
           ++number_nodes;
         }
@@ -613,18 +633,6 @@ CandidateFilter::ResultType CandidateFilter::FilterCandidateInternal(
       return CandidateFilter::BAD_CANDIDATE;
     }
   }
-
-  if (is_strict_mode) {
-    // Filter candidates:
-    // 1) which have the different Pos structure with the top candidate, and
-    // 2) which have atypical Pos structure
-    if (!IsSameNodeStructure(top_nodes, nodes) &&
-        !IsTypicalNodeStructure(*pos_matcher_, nodes)) {
-      MOZC_CANDIDATE_LOG(candidate, "is_strict_mode");
-      return CandidateFilter::BAD_CANDIDATE;
-    }
-  }
-
   return CandidateFilter::GOOD_CANDIDATE;
 }
 
