@@ -32,12 +32,13 @@
 #include <cstdint>
 #include <memory>
 
-#include "base/logging.h"
-#include "base/protobuf/repeated_field.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/random/random.h"
+#include "base/protobuf/repeated_ptr_field.h"
 #include "dictionary/user_dictionary_session.h"
 #include "dictionary/user_dictionary_util.h"
 #include "protocol/user_dictionary_storage.pb.h"
-#include "absl/random/random.h"
 
 namespace mozc {
 namespace user_dictionary {
@@ -538,11 +539,11 @@ UserDictionarySession *UserDictionarySessionHandler::GetSession(
 
 uint64_t UserDictionarySessionHandler::CreateNewSessionId() const {
   uint64_t id = kInvalidSessionId;
+  absl::BitGen bitgen;
   while (true) {
-    id = absl::Uniform<uint64_t>(bitgen_);
+    id = absl::Uniform<uint64_t>(bitgen);
 
-    if (id != kInvalidSessionId &&
-        (session_ == nullptr || session_id_ != id)) {
+    if (id != kInvalidSessionId && (session_ == nullptr || session_id_ != id)) {
       // New id is generated.
       break;
     }
