@@ -39,11 +39,19 @@
 #include <string>
 #include <vector>
 
+#include "absl/container/btree_set.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_join.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "base/config_file_stream.h"
 #include "base/file_util.h"
-#include "base/logging.h"
 #include "base/number_util.h"
 #include "base/util.h"
+#include "base/vlog.h"
 #include "config/character_form_manager.h"
 #include "converter/segments.h"
 #include "dictionary/pos_group.h"
@@ -53,12 +61,6 @@
 #include "rewriter/variants_rewriter.h"
 #include "storage/lru_storage.h"
 #include "transliteration/transliteration.h"
-#include "usage_stats/usage_stats.h"
-#include "absl/container/btree_set.h"
-#include "absl/status/status.h"
-#include "absl/strings/str_cat.h"
-#include "absl/strings/str_join.h"
-#include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace {
@@ -80,11 +82,17 @@ constexpr size_t kMaxRerankSize = 5;
 
 constexpr char kFileName[] = "user://segment.db";
 
+constexpr size_t kRevertCacheSize = 16;
+
 bool IsNumberStyleLearningEnabled(const ConversionRequest &request) {
   // Enabled in mobile (software keyboard & hardware keyboard)
   return request.request().kana_modifier_insensitive_conversion();
 }
 
+bool UseInnerSegments(const ConversionRequest &request) {
+  return request.request().mixed_conversion();
+}
+
 class FeatureValue {
  public:
   FeatureValue() : feature_type_(1), reserved_(0) {}
@@ -129,9 +137,8 @@ inline int GetDefaultCandidateIndex(const Segment &segment) {
       return i;
     }
   }
-  VLOG(2) << "Cannot find default candidate. "
-          << "key: " << segment.key() << ", "
-          << "candidates_size: " << segment.candidates_size();
+  MOZC_VLOG(2) << "Cannot find default candidate. " << "key: " << segment.key()
+               << ", " << "candidates_size: " << segment.candidates_size();
   return 0;
 }
 
@@ -245,7 +252,7 @@ std::string FeatureKey::Current(absl::string_view base_key,
 // Feature "Single"
 std::string FeatureKey::Single(absl::string_view base_key,
                                absl::string_view base_value) const {
-  if (segments_.segments_size() - segments_.history_segments_size() != 1) {
+  if (segments_.conversion_segments_size() != 1) {
     return "";
   }
   return StrJoinWithTabs("S", base_key, base_value);
@@ -291,13 +298,13 @@ std::string FeatureKey::Number(uint16_t type) {
   return StrJoinWithTabs("N", absl::StrCat(type));
 }
 
-bool IsNumberSegment(const Segment &seg) {
-  if (seg.key().empty()) {
+bool IsNumberSegment(const Segment &segment) {
+  if (segment.key().empty()) {
     return false;
   }
   bool is_number = true;
-  for (size_t i = 0; i < seg.key().size(); ++i) {
-    if (!isdigit(static_cast<unsigned char>(seg.key()[i]))) {
+  for (size_t i = 0; i < segment.key().size(); ++i) {
+    if (!isdigit(static_cast<unsigned char>(segment.key()[i]))) {
       is_number = false;
       break;
     }
@@ -309,9 +316,9 @@ void GetValueByType(const Segment *segment,
                     NumberUtil::NumberString::Style style,
                     std::string *output) {
   DCHECK(output);
-  for (size_t i = 0; i < segment->candidates_size(); ++i) {
-    if (segment->candidate(i).style == style) {
-      *output = segment->candidate(i).value;
+  for (const Segment::Candidate *candidate : segment->candidates()) {
+    if (candidate->style == style) {
+      *output = candidate->value;
       return;
     }
   }
@@ -388,13 +395,14 @@ bool GetSameValueCandidatePosition(const Segment *segment,
 }
 
 bool IsT13NCandidate(const Segment::Candidate &cand) {
-  // Regard the cand with 0-id as the transliterated candidate.
+  // The cand with 0-id can be the transliterated candidate.
   return (cand.lid == 0 && cand.rid == 0);
 }
+
 }  // namespace
 
 bool UserSegmentHistoryRewriter::SortCandidates(
-    const std::vector<ScoreCandidate> &sorted_scores, Segment *segment) const {
+    absl::Span<const ScoreCandidate> sorted_scores, Segment *segment) const {
   const uint32_t top_score = sorted_scores[0].score;
   const size_t size = std::min(sorted_scores.size(), kMaxRerankSize);
   constexpr uint32_t kScoreGap = 20;  // TODO(taku): no justification
@@ -472,10 +480,11 @@ bool UserSegmentHistoryRewriter::SortCandidates(
 }
 
 UserSegmentHistoryRewriter::UserSegmentHistoryRewriter(
-    const PosMatcher *pos_matcher, const PosGroup *pos_group)
+    const PosMatcher &pos_matcher, const PosGroup &pos_group)
     : storage_(std::make_unique<LruStorage>()),
-      pos_matcher_(pos_matcher),
-      pos_group_(pos_group) {
+      pos_matcher_(&pos_matcher),
+      pos_group_(&pos_group),
+      revert_cache_(kRevertCacheSize) {
   Reload();
 
   CHECK_EQ(sizeof(uint32_t), sizeof(FeatureValue));
@@ -483,16 +492,17 @@ UserSegmentHistoryRewriter::UserSegmentHistoryRewriter(
 }
 
 UserSegmentHistoryRewriter::Score UserSegmentHistoryRewriter::GetScore(
-    const Segments &segments, size_t segment_index, int candidate_index) const {
+    const ConversionRequest &request, const Segments &segments,
+    size_t segment_index, int candidate_index) const {
   const size_t segments_size = segments.conversion_segments_size();
   const Segment::Candidate &top_candidate =
       segments.segment(segment_index).candidate(0);
   const Segment::Candidate &candidate =
       segments.segment(segment_index).candidate(candidate_index);
-  const std::string &all_value = candidate.value;
-  const std::string &content_value = candidate.content_value;
-  const std::string &all_key = segments.segment(segment_index).key();
-  const std::string &content_key = candidate.content_key;
+  absl::string_view all_value = candidate.value;
+  absl::string_view content_value = candidate.content_value;
+  absl::string_view all_key = segments.segment(segment_index).key();
+  absl::string_view content_key = candidate.content_key;
   // if the segments are resized by user OR
   // either top/target candidate has CONTEXT_SENSITIVE flags,
   // don't apply UNIGRAM model
@@ -521,7 +531,7 @@ UserSegmentHistoryRewriter::Score UserSegmentHistoryRewriter::GetScore(
   score.Update(Fetch(fkey.RightNumber(content_key, content_value),
                      bigram_number_weight));
 
-  const bool is_replaceable = Replaceable(top_candidate, candidate);
+  const bool is_replaceable = Replaceable(request, top_candidate, candidate);
   if (!context_sensitive && is_replaceable) {
     score.Update(Fetch(fkey.Current(all_key, all_value), unigram_weight));
   }
@@ -554,19 +564,23 @@ UserSegmentHistoryRewriter::Score UserSegmentHistoryRewriter::GetScore(
   return score;
 }
 
-// Returns true if |lhs| candidate can be replaceable with |rhs|.
+// Returns true if |best_candidate| can be replaceable with |target_candidate|.
+// Here, "best candidate" means the candidate from converter before applying
+// personalization.
 bool UserSegmentHistoryRewriter::Replaceable(
-    const Segment::Candidate &lhs, const Segment::Candidate &rhs) const {
-  const bool same_functional_value =
-      (lhs.functional_value() == rhs.functional_value());
-  const bool same_pos_group =
-      (pos_group_->GetPosGroup(lhs.lid) == pos_group_->GetPosGroup(rhs.lid));
+    const ConversionRequest &request, const Segment::Candidate &best_candidate,
+    const Segment::Candidate &target_candidate) const {
+  const bool same_functional_value = (best_candidate.functional_value() ==
+                                      target_candidate.functional_value());
+  const bool same_pos_group = (pos_group_->GetPosGroup(best_candidate.lid) ==
+                               pos_group_->GetPosGroup(target_candidate.lid));
   return (same_functional_value &&
-          (same_pos_group || IsT13NCandidate(lhs) || IsT13NCandidate(rhs)));
+          (same_pos_group || IsT13NCandidate(best_candidate) ||
+           IsT13NCandidate(target_candidate)));
 }
 
 void UserSegmentHistoryRewriter::RememberNumberPreference(
-    const Segment &segment) {
+    const Segment &segment, std::vector<std::string> &revert_entries) {
   const Segment::Candidate &candidate = segment.candidate(0);
 
   if ((candidate.style ==
@@ -583,20 +597,18 @@ void UserSegmentHistoryRewriter::RememberNumberPreference(
     // However, access time is count by second, so
     // separated and default is learned at same time
     // This problem is solved by workaround on lookup.
-    Insert(FeatureKey::Number(NumberUtil::NumberString::DEFAULT_STYLE), true);
+    Insert(FeatureKey::Number(NumberUtil::NumberString::DEFAULT_STYLE), true,
+           revert_entries);
   }
 
   // Always insert for numbers
-  Insert(FeatureKey::Number(candidate.style), true);
+  Insert(FeatureKey::Number(candidate.style), true, revert_entries);
 }
 
 void UserSegmentHistoryRewriter::RememberFirstCandidate(
-    const Segments &segments, size_t segment_index) {
+    const ConversionRequest &request, const Segments &segments,
+    size_t segment_index, std::vector<std::string> &revert_entries) {
   const Segment &seg = segments.segment(segment_index);
-  if (seg.candidates_size() <= 1) {
-    return;
-  }
-
   const Segment::Candidate &candidate = seg.candidate(0);
 
   // http://b/issue?id=3156109
@@ -608,10 +620,10 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   const bool context_sensitive =
       segments.resized() ||
       (candidate.attributes & Segment::Candidate::CONTEXT_SENSITIVE);
-  const std::string &all_value = candidate.value;
-  const std::string &content_value = candidate.content_value;
-  const std::string &all_key = seg.key();
-  const std::string &content_key = candidate.content_key;
+  absl::string_view all_value = candidate.value;
+  absl::string_view content_value = candidate.content_value;
+  absl::string_view all_key = seg.key();
+  absl::string_view content_key = candidate.content_key;
 
   // even if the candidate was the top (default) candidate,
   // ERANKED will be set when user changes the ranking
@@ -619,39 +631,47 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
       ((candidate.attributes & Segment::Candidate::RERANKED) != 0);
 
   // Compare the POS group and Functional value.
-  // if "is_replaceable" is true, it means that  the target candidate can
-  // "SAFELY" be replaceable with the top candidate.
+  // if "is_replaceable_with_top" is true, it means that  the target candidate
+  // can "SAFELY" be replaceable with the top candidate.
   const int top_index = GetDefaultCandidateIndex(seg);
   const bool is_replaceable_with_top =
-      ((top_index == 0) || Replaceable(seg.candidate(top_index), candidate));
-
+      ((top_index == 0) ||
+       Replaceable(request, seg.candidate(top_index), candidate));
   FeatureKey fkey(segments, *pos_matcher_, segment_index);
-  Insert(fkey.LeftRight(all_key, all_value), force_insert);
-  Insert(fkey.LeftLeft(all_key, all_value), force_insert);
-  Insert(fkey.RightRight(all_key, all_value), force_insert);
-  Insert(fkey.Left(all_key, all_value), force_insert);
-  Insert(fkey.Right(all_key, all_value), force_insert);
-  Insert(fkey.LeftNumber(all_key, all_value), force_insert);
-  Insert(fkey.RightNumber(all_key, all_value), force_insert);
-  Insert(fkey.Single(all_key, all_value), force_insert);
+  Insert(fkey.LeftRight(all_key, all_value), force_insert, revert_entries);
+  Insert(fkey.LeftLeft(all_key, all_value), force_insert, revert_entries);
+  Insert(fkey.RightRight(all_key, all_value), force_insert, revert_entries);
+  Insert(fkey.Left(all_key, all_value), force_insert, revert_entries);
+  Insert(fkey.Right(all_key, all_value), force_insert, revert_entries);
+  Insert(fkey.LeftNumber(all_key, all_value), force_insert, revert_entries);
+  Insert(fkey.RightNumber(all_key, all_value), force_insert, revert_entries);
+  Insert(fkey.Single(all_key, all_value), force_insert, revert_entries);
 
   if (!context_sensitive && is_replaceable_with_top) {
-    Insert(fkey.Current(all_key, all_value), force_insert);
+    Insert(fkey.Current(all_key, all_value), force_insert, revert_entries);
   }
 
   // save content value
   if (all_value != content_value && all_key != content_key &&
       is_replaceable_with_top) {
-    Insert(fkey.LeftRight(content_key, content_value), force_insert);
-    Insert(fkey.LeftLeft(content_key, content_value), force_insert);
-    Insert(fkey.RightRight(content_key, content_value), force_insert);
-    Insert(fkey.Left(content_key, content_value), force_insert);
-    Insert(fkey.Right(content_key, content_value), force_insert);
-    Insert(fkey.LeftNumber(content_key, content_value), force_insert);
-    Insert(fkey.RightNumber(content_key, content_value), force_insert);
-    Insert(fkey.Single(content_key, content_value), force_insert);
+    Insert(fkey.LeftRight(content_key, content_value), force_insert,
+           revert_entries);
+    Insert(fkey.LeftLeft(content_key, content_value), force_insert,
+           revert_entries);
+    Insert(fkey.RightRight(content_key, content_value), force_insert,
+           revert_entries);
+    Insert(fkey.Left(content_key, content_value), force_insert, revert_entries);
+    Insert(fkey.Right(content_key, content_value), force_insert,
+           revert_entries);
+    Insert(fkey.LeftNumber(content_key, content_value), force_insert,
+           revert_entries);
+    Insert(fkey.RightNumber(content_key, content_value), force_insert,
+           revert_entries);
+    Insert(fkey.Single(content_key, content_value), force_insert,
+           revert_entries);
     if (!context_sensitive) {
-      Insert(fkey.Current(content_key, content_value), force_insert);
+      Insert(fkey.Current(content_key, content_value), force_insert,
+             revert_entries);
     }
   }
 
@@ -660,34 +680,35 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   absl::string_view close_bracket_value;
   if (Util::IsOpenBracket(content_key, &close_bracket_key) &&
       Util::IsOpenBracket(content_value, &close_bracket_value)) {
-    Insert(fkey.Single(close_bracket_key, close_bracket_value), force_insert);
+    Insert(fkey.Single(close_bracket_key, close_bracket_value), force_insert,
+           revert_entries);
     if (!context_sensitive) {
-      Insert(fkey.Current(close_bracket_key, close_bracket_value),
-             force_insert);
+      Insert(fkey.Current(close_bracket_key, close_bracket_value), force_insert,
+             revert_entries);
     }
   }
 }
 
 bool UserSegmentHistoryRewriter::IsAvailable(const ConversionRequest &request,
                                              const Segments &segments) const {
-  if (request.config().incognito_mode()) {
-    VLOG(2) << "incognito_mode";
+  if (request.incognito_mode()) {
+    MOZC_VLOG(2) << "incognito_mode";
     return false;
   }
 
   if (!request.enable_user_history_for_conversion()) {
-    VLOG(2) << "user history for conversion is disabled";
+    MOZC_VLOG(2) << "user history for conversion is disabled";
     return false;
   }
 
   if (storage_ == nullptr) {
-    VLOG(2) << "storage is NULL";
+    MOZC_VLOG(2) << "storage is NULL";
     return false;
   }
 
   // check that all segments have candidate
-  for (size_t i = 0; i < segments.segments_size(); ++i) {
-    if (segments.segment(i).candidates_size() == 0) {
+  for (const Segment &segment : segments) {
+    if (segment.candidates_size() == 0) {
       LOG(ERROR) << "candidate size is 0";
       return false;
     }
@@ -696,24 +717,81 @@ bool UserSegmentHistoryRewriter::IsAvailable(const ConversionRequest &request,
   return true;
 }
 
+// Returns segments for learning.
+// Inner segments boundary will be expanded.
+Segments UserSegmentHistoryRewriter::MakeLearningSegmentsFromInnerSegments(
+    const ConversionRequest &request, const Segments &segments) {
+  auto inner_segments_info_available = [&request](const Segment::Candidate &c) {
+    if (request.request()
+            .decoder_experiment_params()
+            .apply_single_inner_segment_boundary()) {
+      return !c.inner_segment_boundary.empty();
+    } else {
+      return c.inner_segment_boundary.size() > 1;
+    }
+  };
+
+  Segments ret;
+  for (const Segment &segment : segments) {
+    const Segment::Candidate &candidate = segment.candidate(0);
+    if (!inner_segments_info_available(candidate)) {
+      // No inner segment info
+      Segment *seg = ret.add_segment();
+      *seg = segment;
+      continue;
+    }
+    for (Segment::Candidate::InnerSegmentIterator iter(&candidate);
+         !iter.Done(); iter.Next()) {
+      size_t index = iter.GetIndex();
+      absl::string_view key = iter.GetKey();
+
+      Segment *seg = ret.add_segment();
+      seg->set_segment_type(segment.segment_type());
+      seg->set_key(key);
+      seg->clear_candidates();
+
+      Segment::Candidate *cand = seg->add_candidate();
+      cand->attributes = candidate.attributes;
+      cand->key = key;
+      cand->content_key = iter.GetContentKey();
+      cand->value = iter.GetValue();
+      cand->content_value = iter.GetContentValue();
+      // Fill IDs for the first and last inner segment.
+      if (index == 0) {
+        cand->lid = candidate.lid;
+        cand->rid = candidate.lid;
+      } else if (index == candidate.inner_segment_boundary.size() - 1) {
+        cand->lid = candidate.rid;
+        cand->rid = candidate.rid;
+      }
+    }
+  }
+  return ret;
+}
+
 void UserSegmentHistoryRewriter::Finish(const ConversionRequest &request,
-                                        Segments *segments) {
+                                        const Segments &segments) {
   if (request.request_type() != ConversionRequest::CONVERSION) {
     return;
   }
 
-  if (!IsAvailable(request, *segments)) {
+  if (!IsAvailable(request, segments)) {
     return;
   }
 
   if (request.config().history_learning_level() != Config::DEFAULT_HISTORY) {
-    VLOG(2) << "history_learning_level is not DEFAULT_HISTORY";
+    MOZC_VLOG(2) << "history_learning_level is not DEFAULT_HISTORY";
     return;
   }
 
-  for (size_t i = segments->history_segments_size();
-       i < segments->segments_size(); ++i) {
-    const Segment &segment = segments->segment(i);
+  const Segments target_segments =
+      UseInnerSegments(request)
+          ? MakeLearningSegmentsFromInnerSegments(request, segments)
+          : segments;
+  std::vector<std::string> revert_entries;
+  for (size_t i = target_segments.history_segments_size();
+       i < target_segments.segments_size(); ++i) {
+    const Segment &segment = target_segments.segment(i);
     if (segment.candidates_size() <= 0 ||
         segment.segment_type() != Segment::FIXED_VALUE ||
         segment.candidate(0).attributes &
@@ -721,15 +799,14 @@ void UserSegmentHistoryRewriter::Finish(const ConversionRequest &request,
       continue;
     }
     if (IsNumberSegment(segment) && !IsNumberStyleLearningEnabled(request)) {
-      RememberNumberPreference(segment);
+      RememberNumberPreference(segment, revert_entries);
       continue;
     }
     InsertTriggerKey(segment);
-    RememberFirstCandidate(*segments, i);
+    RememberFirstCandidate(request, target_segments, i, revert_entries);
   }
-  // update usage stats here
-  usage_stats::UsageStats::SetInteger("UserSegmentHistoryEntrySize",
-                                      static_cast<int>(storage_->used_size()));
+
+  revert_cache_.Insert(segments.revert_id(), revert_entries);
 }
 
 bool UserSegmentHistoryRewriter::Sync() {
@@ -791,7 +868,7 @@ bool UserSegmentHistoryRewriter::ShouldRewrite(
 
 void UserSegmentHistoryRewriter::InsertTriggerKey(const Segment &segment) {
   if (!(segment.candidate(0).attributes & Segment::Candidate::RERANKED)) {
-    VLOG(2) << "InsertTriggerKey is skipped";
+    MOZC_VLOG(2) << "InsertTriggerKey is skipped";
     return;
   }
 
@@ -861,16 +938,14 @@ bool UserSegmentHistoryRewriter::Rewrite(const ConversionRequest &request,
   }
 
   if (request.config().history_learning_level() == Config::NO_HISTORY) {
-    VLOG(2) << "history_learning_level is NO_HISTORY";
+    MOZC_VLOG(2) << "history_learning_level is NO_HISTORY";
     return false;
   }
 
   // set BEST_CANDIDATE marker in advance
-  for (size_t i = 0; i < segments->segments_size(); ++i) {
-    Segment *segment = segments->mutable_segment(i);
-    DCHECK(segment);
-    DCHECK_GT(segment->candidates_size(), 0);
-    segment->mutable_candidate(0)->attributes |=
+  for (Segment &segment : *segments) {
+    DCHECK_GT(segment.candidates_size(), 0);
+    segment.mutable_candidate(0)->attributes |=
         Segment::Candidate::BEST_CANDIDATE;
   }
 
@@ -904,7 +979,8 @@ bool UserSegmentHistoryRewriter::Rewrite(const ConversionRequest &request,
     }
 
     if (segment->candidates_size() < max_candidates_size) {
-      DVLOG(2) << "Cannot expand candidates. ignored. Rewrite may be failed";
+      MOZC_DVLOG(2)
+          << "Cannot expand candidates. ignored. Rewrite may be failed";
     }
 
     // for each all candidates expanded
@@ -918,7 +994,7 @@ bool UserSegmentHistoryRewriter::Rewrite(const ConversionRequest &request,
                               transliteration::NUM_T13N_TYPES);
       }
 
-      const Score score = GetScore(*segments, i, j);
+      const Score score = GetScore(request, *segments, i, j);
       if (score.score > 0) {
         scores.emplace_back(score, &segment->candidate(j));
       }
@@ -931,17 +1007,58 @@ bool UserSegmentHistoryRewriter::Rewrite(const ConversionRequest &request,
     std::stable_sort(scores.begin(), scores.end(),
                      std::greater<ScoreCandidate>());
     modified |= SortCandidates(scores, segment);
+    if (!(segment->candidate(0).attributes &
+          Segment::Candidate::BEST_CANDIDATE)) {
+      segment->mutable_candidate(0)->attributes |=
+          Segment::Candidate::USER_SEGMENT_HISTORY_REWRITER;
+    }
   }
   return modified;
 }
 
 void UserSegmentHistoryRewriter::Clear() {
   if (storage_ != nullptr) {
-    VLOG(1) << "Clearing user segment data";
+    MOZC_VLOG(1) << "Clearing user segment data";
     storage_->Clear();
   }
 }
 
+void UserSegmentHistoryRewriter::Revert(const Segments &segments) {
+  const std::vector<std::string> *revert_entries =
+      revert_cache_.LookupWithoutInsert(segments.revert_id());
+  if (!revert_entries) {
+    return;
+  }
+  for (const auto &key : *revert_entries) {
+    MOZC_VLOG(2) << "Erasing the key: " << key;
+    storage_->Delete(key);
+  }
+}
+
+bool UserSegmentHistoryRewriter::ClearHistoryEntry(const Segments &segments,
+                                                   size_t segment_index,
+                                                   int candidate_index) {
+  DCHECK_LT(segment_index, segments.segments_size());
+  const Segment &segment = segments.segment(segment_index);
+  DCHECK(segment.is_valid_index(candidate_index));
+  const Segment::Candidate &candidate = segment.candidate(0);
+  absl::string_view key = candidate.key;
+  absl::string_view value = candidate.value;
+
+  FeatureKey fkey(segments, *pos_matcher_, segment_index);
+  bool result = false;
+  result |= DeleteEntry(fkey.LeftRight(key, value));
+  result |= DeleteEntry(fkey.LeftLeft(key, value));
+  result |= DeleteEntry(fkey.RightRight(key, value));
+  result |= DeleteEntry(fkey.Left(key, value));
+  result |= DeleteEntry(fkey.Right(key, value));
+  result |= DeleteEntry(fkey.LeftNumber(key, value));
+  result |= DeleteEntry(fkey.RightNumber(key, value));
+  result |= DeleteEntry(fkey.Single(key, value));
+  result |= DeleteEntry(fkey.Current(key, value));
+  return result;
+}
+
 bool UserSegmentHistoryRewriter::IsPunctuation(
     const Segment &seg, const Segment::Candidate &candidate) const {
   return (pos_matcher_->IsJapanesePunctuations(candidate.lid) &&
@@ -962,16 +1079,40 @@ UserSegmentHistoryRewriter::Score UserSegmentHistoryRewriter::Fetch(
   return {0, 0};
 }
 
-void UserSegmentHistoryRewriter::Insert(absl::string_view key, bool force) {
-  if (!key.empty()) {
-    FeatureValue v;
-    DCHECK(v.IsValid());
-    if (force) {
-      storage_->Insert(key, reinterpret_cast<const char *>(&v));
-    } else {
-      storage_->TryInsert(key, reinterpret_cast<const char *>(&v));
-    }
+void UserSegmentHistoryRewriter::Insert(
+    absl::string_view key, bool force,
+    std::vector<std::string> &revert_entries) {
+  if (key.empty()) {
+    return;
+  }
+
+  MaybeInsertRevertEntry(key, revert_entries);
+
+  FeatureValue v;
+  DCHECK(v.IsValid());
+  if (force) {
+    storage_->Insert(key, reinterpret_cast<const char *>(&v));
+  } else {
+    storage_->TryInsert(key, reinterpret_cast<const char *>(&v));
   }
 }
 
+void UserSegmentHistoryRewriter::MaybeInsertRevertEntry(
+    absl::string_view key, std::vector<std::string> &revert_entries) {
+  if (storage_->Lookup(key) != nullptr) {
+    return;
+  }
+
+  revert_entries.emplace_back(key);
+}
+
+bool UserSegmentHistoryRewriter::DeleteEntry(absl::string_view key) {
+  if (storage_->Lookup(key) == nullptr) {
+    return false;
+  }
+  MOZC_VLOG(2) << "Erasing the key: " << key;
+  storage_->Delete(key);
+  return true;
+}
+
 }  // namespace mozc
