@@ -29,28 +29,26 @@
 
 #include "win32/tip/tip_lang_bar_menu.h"
 
-// clang-format off
 #include <atlbase.h>
 #include <atltypes.h>
-#include <atlapp.h>
-#include <atlmisc.h>
-#include <atlgdi.h>
-#include <atluser.h>
+#include <commctrl.h>
 #include <ctfutb.h>
 #include <ole2.h>
 #include <olectl.h>
 #include <strsafe.h>
-// clang-format on
 #include <wil/com.h>
+#include <wil/resource.h>
+#include <windows.h>
+#include <winuser.h>
 
 #include <cstddef>
 #include <string>
 #include <utility>
 
-#include "base/win32/com.h"
-#include "base/win32/com_implements.h"
 #include "absl/base/casts.h"
 #include "absl/base/macros.h"
+#include "base/win32/com.h"
+#include "base/win32/com_implements.h"
 #include "win32/base/text_icon.h"
 #include "win32/base/tsf_profile.h"
 #include "win32/tip/tip_dll_module.h"
@@ -68,12 +66,6 @@ bool IsIIDOf<ITfLangBarItemButton>(REFIID riid) {
 namespace tsf {
 namespace {
 
-using WTL::CBitmap;
-using WTL::CDC;
-using WTL::CIcon;
-using WTL::CMenu;
-using WTL::CMenuItemInfo;
-
 // Represents the cookie for the sink to a TipLangBarButton object.
 constexpr int kTipLangBarMenuCookie =
     (('M' << 24) | ('o' << 16) | ('z' << 8) | ('c' << 0));
@@ -118,43 +110,33 @@ HICON LoadIconFromResource(HINSTANCE instance, UINT icon_id) {
                                         LR_CREATEDIBSECTION));
 }
 
+class IconBitmaps {
+ public:
+  IconBitmaps() = default;
+  IconBitmaps(IconBitmaps &&) = default;
+
+  explicit IconBitmaps(const ICONINFO &info)
+      : color_(info.hbmColor), mask_(info.hbmMask) {}
+
+  wil::unique_hbitmap color_;
+  wil::unique_hbitmap mask_;
+};
+
 // Retrieves the bitmap handle loaded by using an icon ID.
-// Returns true if the specified icons is available as bitmaps.
-// Caller can set nullptr for |color| and/or |mask| to represent not to receive
-// the specified handle even if it exists.  Caller should releases any returned
-// bitmap handle and this function releases any handle which is not received
-// by the caller.
-bool LoadIconAsBitmap(HINSTANCE instance, UINT icon_id_for_non_theme,
-                      UINT icon_id_for_theme, HBITMAP *color, HBITMAP *mask) {
-  if (color != nullptr) {
-    *color = nullptr;
-  }
-  if (mask != nullptr) {
-    *mask = nullptr;
-  }
+IconBitmaps LoadIconBitmaps(HINSTANCE instance, UINT icon_id_for_non_theme,
+                            UINT icon_id_for_theme) {
+  wil::unique_hicon icon(LoadIconFromResource(instance, icon_id_for_theme));
 
-  CIcon icon(LoadIconFromResource(instance, icon_id_for_theme));
-
-  if (icon.IsNull()) {
-    return false;
+  if (!icon.is_valid()) {
+    return {};
   }
 
   ICONINFO icon_info = {};
-  if (!::GetIconInfo(icon, &icon_info)) {
-    return false;
+  if (!::GetIconInfo(icon.get(), &icon_info)) {
+    return {};
   }
 
-  CBitmap color_bitmap(icon_info.hbmColor);
-  CBitmap mask_bitmap(icon_info.hbmMask);
-
-  if (color != nullptr) {
-    *color = color_bitmap.Detach();
-  }
-  if (mask != nullptr) {
-    *mask = mask_bitmap.Detach();
-  }
-
-  return true;
+  return IconBitmaps(icon_info);
 }
 
 }  // namespace
@@ -168,7 +150,6 @@ HRESULT TipLangBarMenuDataArray::Init(HINSTANCE instance,
   for (int i = 0; i < count; ++i) {
     TipLangBarMenuData data = {};
     int length = 0;
-    HICON icon = nullptr;
     if ((menu[i].flags_ & TF_LBMENUF_SEPARATOR) == 0) {
       // Retrieve the menu text and button icon.
       length = ::LoadString(instance, menu[i].text_id_, &data.text_[0],
@@ -275,13 +256,12 @@ STDMETHODIMP TipLangBarButton::OnClick(TfLBIClick click, POINT point,
     return S_OK;
   }
 
-  CMenu menu;
-  menu.CreatePopupMenu();
+  wil::unique_hmenu menu(::CreatePopupMenu());
   for (size_t i = 0; i < menu_data_size(); ++i) {
     TipLangBarMenuData *data = menu_data(i);
     const UINT id = static_cast<UINT>(data->item_id_);
-    const wchar_t *text = data->text_;
-    CMenuItemInfo info;
+    MENUITEMINFO info = {};
+    info.cbSize = CCSIZEOF_STRUCT(MENUITEMINFO, hbmpItem);
     if (data->flags_ == TF_LBMENUF_SEPARATOR) {
       info.fMask |= MIIM_FTYPE;
       info.fType |= MFT_SEPARATOR;
@@ -319,7 +299,7 @@ STDMETHODIMP TipLangBarButton::OnClick(TfLBIClick click, POINT point,
           break;
       }
     }
-    menu.InsertMenuItemW(i, TRUE, &info);
+    ::InsertMenuItem(menu.get(), i, TRUE, &info);
   }
 
   // Caveats: TPM_NONOTIFY is important because the attached window may
@@ -327,8 +307,22 @@ STDMETHODIMP TipLangBarButton::OnClick(TfLBIClick click, POINT point,
   // from this issue with Internet Explorer 10 on Windows 8. b/10217103.
   constexpr DWORD kMenuFlags = TPM_NONOTIFY | TPM_RETURNCMD | TPM_LEFTALIGN |
                                TPM_TOPALIGN | TPM_LEFTBUTTON;
-  const BOOL result =
-      menu.TrackPopupMenu(kMenuFlags, point.x, point.y, ::GetFocus());
+
+  // Make sure that the x-coordinate is inside the work-area.
+  const HMONITOR monitor_handle =
+      ::MonitorFromPoint(point, MONITOR_DEFAULTTONEAREST);
+  if (monitor_handle != nullptr) {
+    MONITORINFO monitor_info = {};
+    monitor_info.cbSize = CCSIZEOF_STRUCT(MONITORINFO, dwFlags);
+    if (::GetMonitorInfo(monitor_handle, &monitor_info)) {
+      point.x = std::clamp(point.x,
+                           monitor_info.rcWork.left,
+                           monitor_info.rcWork.right);
+    }
+  }
+
+  const BOOL result = ::TrackPopupMenu(menu.get(), kMenuFlags, point.x, point.y,
+                                       0, ::GetFocus(), nullptr);
   if (!result) {
     return E_FAIL;
   }
@@ -388,7 +382,6 @@ STDMETHODIMP TipLangBarButton::UnadviseSink(DWORD cookie) {
 // button menu to a language bar.
 HRESULT TipLangBarButton::Init(HINSTANCE instance, int string_id,
                                const TipLangBarMenuItem *menu, int count) {
-  HRESULT result = S_OK;
   // Retrieve the text label from the resource.
   // This string is also used as a tool-tip text.
   ::LoadString(instance, string_id, &item_info_.szDescription[0],
@@ -427,9 +420,10 @@ bool TipLangBarButton::CanContextMenuDisplay32bppIcon() {
   // unless the current display mode is 32-bpp.  We cannot assume we can
   // display a 32-bpp icon for a context menu icon on the LangBar unless the
   // current display mode is 32-bpp.  See http://b/2260057
-  CDC display_dc(::GetDC(nullptr));
-  return !display_dc.IsNull() && display_dc.GetDeviceCaps(PLANES) == 1 &&
-         display_dc.GetDeviceCaps(BITSPIXEL) == 32;
+  wil::unique_hdc memory_dc(::GetDC(nullptr));
+  return memory_dc.is_valid() &&
+         ::GetDeviceCaps(memory_dc.get(), PLANES) == 1 &&
+         ::GetDeviceCaps(memory_dc.get(), BITSPIXEL) == 32;
 }
 
 TipLangBarMenuData *TipLangBarButton::menu_data(size_t i) {
@@ -476,14 +470,11 @@ STDMETHODIMP TipLangBarMenuButton::InitMenu(ITfMenu *menu) {
     const UINT icon_id_for_theme = CanContextMenuDisplay32bppIcon()
                                        ? data->icon_id_for_theme_
                                        : data->icon_id_for_non_theme_;
-    CBitmap bitmap;
-    CBitmap mask;
-    // If LoadIconAsBitmap fails, |bitmap.m_hBitmap| and |mask.m_hBitmap|
-    // remain nullptr bitmap handles.
-    LoadIconAsBitmap(TipDllModule::module_handle(),
-                     data->icon_id_for_non_theme_, icon_id_for_theme,
-                     &bitmap.m_hBitmap, &mask.m_hBitmap);
-    result = menu->AddMenuItem(i, data->flags_, bitmap, mask, data->text_,
+    const IconBitmaps icon_bitmaps =
+        LoadIconBitmaps(TipDllModule::module_handle(),
+                        data->icon_id_for_non_theme_, icon_id_for_theme);
+    result = menu->AddMenuItem(i, data->flags_, icon_bitmaps.color_.get(),
+                               icon_bitmaps.mask_.get(), data->text_,
                                data->length_, nullptr);
     if (result != S_OK) {
       break;
@@ -511,7 +502,6 @@ STDMETHODIMP TipLangBarMenuButton::OnMenuSelect(UINT menu_id) {
 // Implements the ITfLangBarItem::GetInfo() function.
 // This function is called by Windows to update this button menu.
 STDMETHODIMP TipLangBarMenuButton::GetInfo(TF_LANGBARITEMINFO *item_info) {
-  HRESULT result = S_OK;
 
   if (item_info == nullptr) {
     return E_INVALIDARG;
@@ -520,20 +510,20 @@ STDMETHODIMP TipLangBarMenuButton::GetInfo(TF_LANGBARITEMINFO *item_info) {
   // Just copies the cached TF_LANGBARITEMINFO object.
   *item_info = *this->item_info();
 
-  CIcon icon;
-  if (FAILED(GetIcon(&icon.m_hIcon)) || icon.IsNull()) {
+  wil::unique_hicon icon;
+  if (FAILED(GetIcon(icon.put())) || !icon.is_valid()) {
     return S_OK;
   }
 
   ICONINFO icon_info = {};
-  const BOOL get_icon_succeeded = icon.GetIconInfo(&icon_info);
-  CBitmap color(icon_info.hbmColor);
-  CBitmap mask(icon_info.hbmMask);
+  const BOOL get_icon_succeeded = ::GetIconInfo(icon.get(), &icon_info);
+  wil::unique_hbitmap color(icon_info.hbmColor);
+  wil::unique_hbitmap mask(icon_info.hbmMask);
   if (!get_icon_succeeded) {
     return S_OK;
   }
 
-  if (color.IsNull() && !mask.IsNull()) {
+  if (!color.is_valid() && mask.is_valid()) {
     item_info->dwStyle |= TF_LBI_STYLE_TEXTCOLORICON;
     return S_OK;
   }
@@ -606,20 +596,20 @@ STDMETHODIMP TipLangBarToggleButton::GetInfo(TF_LANGBARITEMINFO *item_info) {
     return result;
   }
 
-  CIcon icon;
-  if (FAILED(GetIcon(&icon.m_hIcon)) || icon.IsNull()) {
+  wil::unique_hicon icon;
+  if (FAILED(GetIcon(icon.put())) || !icon.is_valid()) {
     return S_OK;
   }
 
   ICONINFO icon_info = {};
-  const BOOL get_icon_succeeded = icon.GetIconInfo(&icon_info);
-  CBitmap color(icon_info.hbmColor);
-  CBitmap mask(icon_info.hbmMask);
+  const BOOL get_icon_succeeded = ::GetIconInfo(icon.get(), &icon_info);
+  wil::unique_hbitmap color(icon_info.hbmColor);
+  wil::unique_hbitmap mask(icon_info.hbmMask);
   if (!get_icon_succeeded) {
     return S_OK;
   }
 
-  if (color.IsNull() && !mask.IsNull()) {
+  if (!color.is_valid() && mask.is_valid()) {
     item_info->dwStyle |= TF_LBI_STYLE_TEXTCOLORICON;
     return S_OK;
   }
@@ -776,14 +766,11 @@ STDMETHODIMP TipSystemLangBarMenu::InitMenu(ITfMenu *menu) {
         TipLangBarButton::CanContextMenuDisplay32bppIcon()
             ? data->icon_id_for_theme_
             : data->icon_id_for_non_theme_;
-    CBitmap bitmap;
-    CBitmap mask;
-    // If LoadIconAsBitmap fails, |bitmap.m_hBitmap| and |mask.m_hBitmap|
-    // remain nullptr bitmap handles.
-    LoadIconAsBitmap(TipDllModule::module_handle(),
-                     data->icon_id_for_non_theme_, icon_id_for_theme,
-                     &bitmap.m_hBitmap, &mask.m_hBitmap);
-    result = menu->AddMenuItem(i, data->flags_, bitmap, mask, data->text_,
+    const IconBitmaps &icon_bitmaps =
+        LoadIconBitmaps(TipDllModule::module_handle(),
+                        data->icon_id_for_non_theme_, icon_id_for_theme);
+    result = menu->AddMenuItem(i, data->flags_, icon_bitmaps.color_.get(),
+                               icon_bitmaps.mask_.get(), data->text_,
                                data->length_, nullptr);
     if (result != S_OK) {
       break;
@@ -806,7 +793,7 @@ STDMETHODIMP TipSystemLangBarMenu::OnMenuSelect(UINT menu_id) {
   if (data->item_id_ == TipLangBarCallback::kCancel) {
     return S_OK;
   }
-  const HRESULT result = lang_bar_callback_->OnMenuSelect(
+  (void)lang_bar_callback_->OnMenuSelect(
       static_cast<TipLangBarCallback::ItemId>(data->item_id_));
   return S_OK;
 }
